NATIVE clear_rstack, regs, jsr, "Initializes an empty register stack.  This is the minimum initialization needed for basic test code, but note that other features may not be initialized (gptr in particular)."
 lda #rstackTop
 sta rptr
 sta pptr
 rts

NATIVE acheron, flow, jsr, "Enter Acheron mode, interpreting bytecodes immediately after the JSR instruction."
 pla
 sta iptr
 pla
 sta iptr+1
 ; JSR puts the byte before the instruction to run onto the stack, so start 1 byte from there.
 jmp mainLoop1

OP native, none, flow, "Shifts to 6502 mode starting at the byte after this instruction.  .X is preloaded with rptr for convenience."
 ldx rptr
 inc iptr
 bne :+
  inc iptr+1
:jmp (iptr)

PSEUDO grow, "imm", regs, "Becomes grow4 or grow8", {" .if (imm >= 1) .and (imm <= 16)",EOL,"   grow4 imm",EOL," .elseif (imm <> 0)",EOL,"   grow8 imm",EOL," .endif"}

OPE grow4, imm4p1, regs, "Grow the register stack by imm4 (1-16) slots.  The resulting r0 becomes the new rP."
 ; .X = rptr + delta, need rptr - delta - 2
 ; so perform -(X+1) -1 + rptr + rptr
 inx
 txa
 eor #$ff ; with carry clear, this is an additional -1 during negation
 adc rptr
 clc
 adc rptr
 sta rptr
 sta pptr
 jmp mainLoop1

OP grow8, nslots, regs, "Grow the register stack by imm8 (signed) slots.  The resulting r0 becomes the new rP."
 lda rptr
 adc (iptr),y
 sta rptr
 sta pptr
 jmp mainLoop2


OP dropc, none, bits, "Discard the most recent carry bit."
 asl cstack
 jmp mainLoop1

OP pushregs, rdimm4, regs, "Push imm4 (1-15) registers starting at rD to the CPU stack."
 decode_rdimm4
 tya ; double .Y for how many bytes to push
 asl
 tay
: lda 0,x
  pha
  inx
  dey
 bne :-
 jmp mainLoop2

OP popregs, rdimm4, regs, "Pop imm4 (1-15) registers starting at rD from the CPU stack."
 decode_rdimm4
 tya
 asl
 tay
 adc pptr ; add the bytes to start restoring from the highest address (overshot by 1, so dex first in the loop)
 tax
: pla
  dex
  sta 0,x
  dey
 bne :-
 jmp mainLoop2
 

OP clrmp, none, regs, "memory(rP) := 0"
 lda #0
 stx :+ +1
:sta (0),y ; effectively sta (x),y  .Y is 1
OP clrmbp, none, regs, "memory(rP) := 0 byte"
 lda #0
 sta (0,x)
 jmp mainLoop1

OP stpmb, rd, regs, "memory(rD) := low byte of rP"
  lda 0,x      ; read rP's value first
  sta zptemp
  decode_rd    ; no shifting, just adding an offset and updating rP
  lda zptemp
  sta (0,x)    ; store into (rD)
  jmp mainLoop2c

OP stpm, rd, regs, "memory(rD) := rP"
 lda 0,x
 sta zptemp
 lda 1,x
 sta zptemp+1
 decode_rd
 lda zptemp
 sta (0,x)
 lda zptemp+1
 stx :+ +1
:sta (0),y
 jmp mainLoop2c

OP ldm, rdra, regs, "rD := memory(rA)"
 decode_rdra
 lda #0
 beq _ldm
OP ldmi, rdraimm8, regs, "rD := memory(rA + imm8)"
 decode_rdra
 sec
_ldm:
 sty selfmod1 +1
 sty selfmod2 +1
 bcc :+
  ldy #1
  lda (iptr),y
:tay
selfmod1:
 lda (0),y
 sta zptemp ; buffer the write as to not fail for rD = rA
 iny
selfmod2:
 lda (0),y
 sta 1,x
 lda zptemp
 sta 0,x
 jmp mainLoop2c ;carry will bump it to 3 for the added imm8

OP stm, rdra, regs, "memory(rA) := rD"
 decode_rdra
 lda #0
 beq _stm
OP stmi, rdraimm8, regs, "memory(rA + imm8) := rD"
 decode_rdra
 sec
_stm:
 sty selfmod3 +1
 sty selfmod4 +1
 bcc :+
  ldy #1
  lda (iptr),y
:tay
 lda 0,x
selfmod3:
 sta (0),y
 iny
 lda 1,x
selfmod4:
 sta (0),y
 jmp mainLoop2c ;carry will bump to 3 for the added imm8

OP nswap, none, bits, "Swap nybbles of the lowest byte of rP."
 lda 0,x
 asl
 adc #0
 asl
 adc #0
 asl
 adc #0
 asl
 adc #0
 sta 0,x
 jmp mainLoop1



OP negatep, none, math, "rP := -rP"
 sec
OP notp, none, bits, "rP := rP ^ $ffff"
 lda 0,x
 eor #$ff
 adc #0
 sta 0,x
 lda 1,x
 eor #$ff
 adc #0
 sta 1,x
 jmp mainLoop0





OP andr, rdra, bits, "rD := rD & rA"
 decode_rdra
 lda 0,x
 and 0,y
 sta 0,x
 lda 1,x
 and 1,y
 sta 1,x
 jmp mainLoop2c

OP orr, rdra, bits, "rD := rD | rA"
 decode_rdra
 lda 0,x
 ora 0,y
 sta 0,x
 lda 1,x
 ora 1,y
 sta 1,x
 jmp mainLoop2c

OP xorr, rdra, bits, "rD := rD ^ rA"
 decode_rdra
 lda 0,x
 eor 0,y
 sta 0,x
 lda 1,x
 eor 1,y
 sta 1,x
 jmp mainLoop2c

