;--------------------------------
; Unembedded parameter encodings

.macro OP name, args, category, desc
 .pushseg
 name:
 .segment "UTABLE"
 .scope
 _entry: .word name
 .if .xmatch(args,none)
   EMIT u, name, category, desc, "", ""
 .elseif .xmatch(args,imm16)
   EMIT u, name, category, desc, "imm16", ", <(imm16), >(imm16)"
 .elseif .xmatch(args,imm8)
   EMIT u, name, category, desc, "imm8", ", imm8"
 .elseif .xmatch(args,rel8)
   EMIT u, name, category, desc, "rel8", ", <((rel8) - * + 1)" ; * is the address of the parameter byte, so +1 to get back to the opcode byte
 .elseif .xmatch(args,imm8rel8)
   EMIT u, name, category, desc, "imm8, rel8", ", imm8, <((rel8) - * + 1)"
 .elseif .xmatch(args,imm16rel8)
   EMIT u, name, category, desc, "imm16, rel8", ", <(imm16), >(imm16), <((rel8) - * + 1)"
 .elseif .xmatch(args,rdimm4)
   EMIT u, name, category, desc, "rD, imm4", ", (((rD)<<4) + (imm4))"
 .elseif .xmatch(args,rdimm4p1)
   EMIT u, name, category, desc, "rD, imm4p1", ", (((rD)<<4) + ((imm4p1) - 1))"
 .elseif .xmatch(args,rdra)
   EMIT u, name, category, desc, "rD, rA", ", (((rD)<<4) + (rA))"
 .elseif .xmatch(args,rard)
   EMIT u, name, category, desc, "rD, rA", ", (((rA)<<4) + (rD))", "Reversed encoding for ease of implementation.  Instruction must set pptr's value itself."
 .elseif .xmatch(args,rdraimm8)
   EMIT u, name, category, desc, "rD, rA, imm8", ", (((rD)<<4) + (rA)), imm8"
 .elseif .xmatch(args,rdraimm16)
   EMIT u, name, category, desc, "rD, rA, imm16", ", (((rD)<<4) + (rA)), <(imm16), >(imm16)"
 .elseif .xmatch(args,rdimm4p1rel8)
   EMIT u, name, category, desc, "rD, imm4p1, rel8", ", (((rD)<<4) + <((imm4p1) - 1)), <((rel8) - * + 2)"
 .elseif .xmatch(args,nslots)
   EMIT u, name, category, desc, "imm8", ", (-((imm8)<<1)) & $ff", "Creates a negative rptr delta from imm4."
 .elseif .xmatch(args,pslots)
   EMIT u, name, category, desc, "imm8", ", ((imm8)<<1)", "Creates a positive rptr delta from imm4."
 .elseif .xmatch(args,rd)
   EMIT u, name, category, desc, "rD", ", ((rD)<<1)", "Add rptr to address the register.  Wastes 4 bits of the parameter byte."
 .elseif .xmatch(args,rdimm8p1)
   EMIT u, name, category, desc, "rD, imm8", ", ((rD)<<1), (imm8)-1"
; Add more parameter encodings for unembedded ops here
 .else
   .error .concat("Unembedded instruction ",.string(name)," has an unknown parameter encoding: ",.string(args))
 .endif
 .endscope
 .popseg
.endmacro


;--------------------------------
; Embedded parameter encodings

.macro OPE name, args, category, desc
 .pushseg
 name:
 .segment "ETABLE"
 .scope
 _entry: .word name
 .if .xmatch(args,rd)
   EMIT e, name, category, desc, "rD", " + (rD)"
 .elseif .xmatch(args,rdimm8)
   EMIT e, name, category, desc, "rD, imm8", " + (rD), imm8"
 .elseif .xmatch(args,rdimm16)
   EMIT e, name, category, desc, "rD, imm16", " + (rD), <(imm16), >(imm16)"
 .elseif .xmatch(args,imm4p1)
   EMIT e, name, category, desc, "imm4p1", " + (imm4p1) - 1"
; Add more parameter encodings for embedded ops here
 .else
   .error .concat("Embedded instruction ",.string(name)," has an unknown parameter encoding: ",.string(args))
 .endif
 .endscope
 .popseg
.endmacro
